unit AreasF;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, SpeedBar, ExtCtrls, Placemnt, Menus,
  StdCtrls, Buttons, ComCtrls, RXShell,

  Mail,
  Radio,
  Commands, IPopup;

type
  TAreasForm = class(TForm)
    MainMenu: TMainMenu;
    NavigationItem: TMenuItem;
    GotoPriorWithUnreadItem: TMenuItem;
    GotoPriorWithUnreadItem2: TMenuItem;
    GotoNextWithUnreadItem: TMenuItem;
    GotoNextWithUnreadItem2: TMenuItem;
    FormStorage: TFormStorage;
    SpeedBar: TSpeedBar;
    PrevWithUnreadSpeedItem: TSpeedItem;
    NextWithUnreadSpeedItem: TSpeedItem;
    SpeedBarPopupMenu: TPopupMenu;
    ConfigureItem: TMenuItem;
    ShowHintItem: TMenuItem;
    HideSpeedBar: TMenuItem;
    HelpItem1: TMenuItem;
    SwitchItem: TMenuItem;
    SpeedBarItem: TMenuItem;
    StatusLineItem: TMenuItem;
    MinimizeItem: TMenuItem;
    FileItem: TMenuItem;
    ExitItem: TMenuItem;
    WindowsItem: TMenuItem;
    N5: TMenuItem;
    SwitchToTopBrowser: TMenuItem;
    WindowListItem: TMenuItem;
    Bevel1: TBevel;
    N2: TMenuItem;
    SwitchToTopEditor: TMenuItem;
    N6: TMenuItem;
    LastAreaItem: TMenuItem;
    FirstAreaItem: TMenuItem;
    NetmailSpeedItem: TSpeedItem;
    OptionsItem: TMenuItem;
    UserItem: TMenuItem;
    EnterNetmailItem: TMenuItem;
    N1: TMenuItem;
    Tree: TTreeView;
    ImageList: TImageList;
    StatusBar: TStatusBar;
    AboutItem: TMenuItem;
    UpdateAreasTick: TTimer;
    UpdateAreasStarter: TTimer;
    NewMailIcon: TRxTrayIcon;
    IndirectPopupMenu: TIndirectPopupMenu;
    procedure AnyNavigationItemClick(Sender: TObject);
    procedure TreeKeyPress(Sender: TObject; var Key: Char);
    procedure FormStorageRestorePlacement(Sender: TObject);
    procedure ShowHintItemClick(Sender: TObject);
    procedure HideSpeedBarClick(Sender: TObject);
    procedure SpeedBarItemClick(Sender: TObject);
    procedure StatusLineItemClick(Sender: TObject);
    procedure ConfigureItemClick(Sender: TObject);
    procedure FormStorageSavePlacement(Sender: TObject);
    procedure EnterSelectedAreaItemClick(Sender: TObject);
    procedure MinimizeItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SwitchToTopBrowserClick(Sender: TObject);
    procedure SwitchToTopEditorClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure NetmailItemClick(Sender: TObject);
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
    procedure FormActivate(Sender: TObject);
    procedure TreeGetSelectedIndex(Sender: TObject; Node: TTreeNode);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure UserItemClick(Sender: TObject);
    procedure WindowListItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure UpdateAreasTickTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure UpdateAreasStarterTimer(Sender: TObject);
    procedure NewMailIconClick(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure CreateParams(var Params: TCreateParams); override;
  private
    TotalUnreadCount: Longint;
    procedure UpdateTotalUnreadCount;

    procedure SetTitle(ATitle: string);

    procedure WMSysCommand(var Message: TWMSysCommand); message WM_SYSCOMMAND;
    procedure ApplicationMinimize;

    procedure ApplicationActivate(Sender: TObject);
    procedure ApplicationDeactivate(Sender: TObject);
    procedure ApplicationMessage(var Msg: TMsg; var Handled: Boolean);

    function WindowHook(var Message: TMessage): Boolean;

    procedure CMBeforeScanningAreas(var Msg: TRadioMessage);
      message CM_BeforeScanningAreas;
    procedure CMNextAreaScanned(var Msg: TRadioMessage);
      message CM_NextAreaScanned;
    procedure CMAfterScanningAreas(var Msg: TRadioMessage);
      message CM_AfterScanningAreas;
    procedure CMAreaCountsChange(var Msg: TRadioMessage);
      message CM_AreaCountsChange;
  private
    AreasLine, AreaLine: TStatusPanel;
  private
    procedure ApplicationHint(Sender: TObject);
  private
    { Private declarations }
    IncSearching: Boolean;
    IncSearchCount: Integer;
    FIncSearchedStr: string;

    procedure SetIncSearchedStr(const AIncSearchedStr: string);
    property IncSearchedStr: string read FIncSearchedStr write SetIncSearchedStr;

    procedure UpdateAreaLine;
    procedure UpdateAreaState(Area: TArea; Node: TTreeNode);
    procedure UpdateAreaNode(Area: TArea);
    procedure StartStopUpdatingArea(Area: TArea);

    procedure UpdateAreas;
    procedure CMAreasChanged(var Msg: TRadioMessage);
      message CM_AreasChanged;

    procedure CMStartUpdatingArea(var Msg: TRadioMessage);
      message CM_StartUpdatingArea;
    procedure CMStopUpdatingArea(var Msg: TRadioMessage);
      message CM_StopUpdatingArea;

    procedure CMCurrentAreaChanged(var Msg: TRadioMessage);
      message CM_CurrentAreaChanged;

    procedure AreaOpenedOrClosed(var Msg: TRadioMessage);
    procedure CMAreaOpened(var Msg: TRadioMessage);
      message CM_AreaOpened;
    procedure CMAreaClosed(var Msg: TRadioMessage);
      message CM_AreaClosed;

    procedure UpdateOptions;
  private
    {ActivateMinimized: Boolean;}
  public
    { Public declarations }
    property Title: string{ read GetTitle} write SetTitle;
  public
    { Public declarations }
    procedure Navigate(Offset: Integer);
  end;

var
  AreasForm: TAreasForm;

implementation

{$R *.DFM}
{$R *.RES}

uses
  StrUtils,
  VCLUtils,

  MKGlobT,

  Common,
  Config,
  Flags,

  UserF,
  AreaF,
  EditorF,
  WndListF,
  AboutF;

{ TAreasForm }

procedure TAreasForm.CreateParams;
begin
  inherited CreateParams(Params);
  Params.WndParent := 0;
end;

procedure TAreasForm.FormCreate(Sender: TObject);
begin
// MainForm
  ReadConfig;

  ShowWindow(Application.Handle,SW_HIDE);

  Application.OnActivate := ApplicationActivate;
  Application.OnDeactivate := ApplicationDeactivate;
  Application.OnMessage := ApplicationMessage;
  Application.HookMainWindow(WindowHook);


  Title := '';

//  if RealCmdShow <> SW_SHOWMINNOACTIVE then
//     z
//  ExecuteAreasUpdateThread;


// AreasForm
  Icon.Handle := LoadIcon(hInstance, 'Z_AREASF_ICON');
  ImageList.ResourceLoad(rtBitmap, 'AREASF_TREEIMAGES', clRed);

  AreasLine := StatusBar.Panels[0];
  AreaLine := StatusBar.Panels[1];

  AreaLine.Text := '';
  AreasLine.Text := ''{Areas.GetInfo };

  UpdateAreas;
end;

procedure TAreasForm.FormDestroy(Sender: TObject);
begin
  Application.UnhookMainWindow(WindowHook);
end;

procedure TAreasForm.NewMailIconClick(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  ActivateWindow(Handle);
end;

procedure TAreasForm.ApplicationMinimize;

  procedure CloseAllFormsExceptSelf;
  var
    I: Integer;
    Form: TForm;
  begin
    with Screen do
      for I := 0 to Pred(FormCount) do
      begin
        Form := Forms[I];
        if Form <> Self then
           Form.Close;
      end;
  end;

begin
  //ShowWindow(Application.Handle,SW_HIDE);

  if CloseQuery then
    CloseAllFormsExceptSelf;

  WindowState := wsMinimized;
//  ShowWindow(Handle, SW_SHOWMINNOACTIVE);   WindowState := wsMinimized;
//    focus,
//        - focus    ,
//     
end;

procedure TAreasForm.WMSysCommand(var Message: TWMSysCommand);
begin
  if Message.CmdType and $FFF0 = SC_MINIMIZE then
    ApplicationMinimize
  else
    inherited;
end;

procedure TAreasForm.ApplicationActivate(Sender: TObject);
begin
end;

procedure TAreasForm.ApplicationDeactivate;
var
  I: Integer;
begin
  with Screen do
    for I := 0 to Pred(FormCount) do
      with Forms[I] do
        if Assigned(OnDeactivate) then
          OnDeactivate(nil){     incsearch};
end;

procedure TAreasForm.ApplicationMessage;
{const
  AltMask = $20000000;}
begin
  with Msg do
  begin
    {
          
      Samantha Lane <slx@anpnt22.anp.ansto.gov.au>        c 08 H 15 05:05
    }
    {if (Message = WM_SYSCHAR) and (ActiveForm <> nil) and (ActiveForm.Menu <> nil) then
    begin
      SendMessage(ActiveForm.Handle, Message, wParam, lParam);
      Handled := True;
    end;

       .WindowHook
    }

    if (Message = WM_SYSCHAR) and (Upcase(Char(WParam)) = 'X') then
    begin
      {if MessageDlg('   Alt+X, ?',
        mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
        Exit; ,    ,   ,
              ,
                ,
          ,    Ctrl+Alt+F12,
           @  .
           -   :( }

      Close;
      Handled := True;
    end;
(*    if (Message = WM_SYSKEYDOWN) and
        (LParam{KeyData} and AltMask <> 0) and
        (Char(WParam{CharCode}) = 'X') then
      begin
        Close;
        Handled := True;
      end;*)
  end;
end;

function TAreasForm.WindowHook;
begin
  Result := False;

  with Message do
    case Msg of
      CM_APPKEYDOWN{  AreasForm.MainMenu   __ . !},
      CM_APPSYSCOMMAND{ AreasForm.MainMenu   . !}: Msg := WM_NULL;
(*      WM_ACTIVATEAPP:
        ActivateMinimized := TWMActivateApp(Message).Active{!;} *)
    end;
end;

procedure TAreasForm.SetTitle;
const
  Goodwin = '';
begin
  if ATitle = '' then
  begin
    if TotalUnreadCount <> 0 then
      ATitle := IntToStr(TotalUnreadCount);
    if TossingInProgress then
      ATitle := '[ ]';
  end;
  if ATitle = ''
  then ATitle := Goodwin
  else ATitle := Format('%s: %s', [Goodwin, ATitle]);

  with NewMailIcon do
  begin
    Active := TotalUnreadCount <> 0;
    Hint := ATitle;
  end;

  Caption := ATitle;
  Application.Title := ATitle;
end;


procedure TAreasForm.UserItemClick(Sender: TObject);
var
  UserForm: TUserForm;
begin
  UserForm := TUserForm.Create(Application);
  with UserForm do
  try
    NameEdit.Text := UserName;
    MainAddressEdit.Text := AddrStr(MainAddress);

    AKAsMemo.Lines.Assign(AKAs);
    NewLetterTemplateMemo.Lines.Assign(NewLetterTemplate);
    QuoteReplyTemplateMemo.Lines.Assign(QuoteReplyTemplate);

    if ShowModal <> mrOK then
      Exit;

    UserName := NameEdit.Text;
    ParseAddr(MainAddressEdit.Text, MainAddress, MainAddress);

    AKAs.Assign(AKAsMemo.Lines);
    NewLetterTemplate.Assign(NewLetterTemplateMemo.Lines);
    QuoteReplyTemplate.Assign(QuoteReplyTemplateMemo.Lines);

    WriteConfig;
  finally
    Free;
  end;
end;

procedure TAreasForm.WindowListItemClick(Sender: TObject);
begin
  with TWindowListForm.Create(Application) do
    try
      with ListBox do {   with  - free}
        if ShowModal = mrOK then
          ActivateWindow((Items.Objects[ItemIndex] as TForm).Handle);
    finally
      Free;
    end;
end;

procedure TAreasForm.AboutItemClick(Sender: TObject);
begin
  with TAboutForm.Create(Application) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TAreasForm.UpdateTotalUnreadCount;
begin
  TotalUnreadCount := Areas.GetTotalUnreadCount;

  Title := '';
end;

procedure TAreasForm.CMAreaCountsChange;
begin
  UpdateTotalUnreadCount;

  UpdateAreaNode(Msg.InfoPtr);
  UpdateAreaLine;
end;

procedure TAreasForm.CMBeforeScanningAreas;
begin
  Title := ' ...';
end;

procedure TAreasForm.CMNextAreaScanned;
begin
  with Areas do
    Title := Format('%d%%', [PercentOf(Succ(IndexOf(TArea(Msg.InfoPtr))), Count)])
end;

procedure TAreasForm.CMAfterScanningAreas;
begin
  UpdateTotalUnreadCount;

  if TotalUnreadCount <> 0 then
    if not Application.Terminated{    } then
      MessageBeep(0);
end;

procedure TAreasForm.UpdateAreasTickTimer(Sender: TObject);
begin
  with Areas do
    if (not Updating) and TossingFlagChanged then
      if TossingInProgress{ toss'}
      then
        Title := ''{       " tossing"}
      else{ toss'}
      begin
        Invalidate;
        Update;
        //ExecuteAreasUpdateThread;
      end;
end;

procedure TAreasForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  I: Integer;
  Form: TForm;
begin
  with Screen do
    for I := 0 to Pred(FormCount) do
    begin
      Form := Forms[I];
      if Form <> Self then
         CanClose := CanClose and Form.CloseQuery;  {  Borland! }
    end;
end;

procedure TAreasForm.ExitItemClick(Sender: TObject);
begin
  AreasForm.Close;
end;

procedure TAreasForm.CMAreasChanged;
begin
  UpdateAreas;
end;

procedure TAreasForm.UpdateAreas;
(*
  procedure AdjustColWidths;
  var
    I: Integer;
    CommentWidth, Proposed: Integer;
    Flag: Boolean;
  begin
    with Tree, Canvas, Areas do
    begin
      CommentWidth := 100;
      for I := 0 to Pred(Count) do
        with TArea(Items[I]) do
        begin
          Proposed := TextWidth(Comment^);
          if Proposed > CommentWidth then
            CommentWidth := Proposed;
        end;

      Font := Tree.Font;
      Font.Style := [fsBold];
      ColWidths[0] := CommentWidth + 4;
      ColWidths[1] := TextWidth('12345')+4;
      ColWidths[2] := ColWidths[1];
      ColWidths[3] := 10000;

      RowCount := Count;
    end;

{    Flag := Areas.Count <> 0;
    Tree.Visible := Flag;
    AreaLine.Visible := Flag;
    AreasLine.Visible := Flag;

    if Flag and Active then
      Tree.SetFocus;}
  end;
*)

  procedure UpdateTree;
  var
    Root, Node: TTreeNode;
    I: Integer;
    Area: TArea;
  begin
    with Tree, Items do
    begin
      BeginUpdate;

      Clear;
      Root := Add(nil, '');
      Selected := Root;
      for I := 0 to Pred(Areas.Count) do
      begin
        Area := TArea(Areas[I]);
        Node := AddChildObject(Root, Area.ID{Comment}, Area);
        Node.ImageIndex := 1;
        Area.InfoPtr := Node;
        UpdateAreaState(Area, Node);
      end;
      Root.Expand(False);
      Navigate(+1);

      EndUpdate;
    end;
  end;

  procedure UpdateAreasLine;
  begin
    AreasLine.Text := Areas.GetInfo;
  end;

begin
  UpdateTree;

  UpdateAreasLine;
  UpdateAreaLine;
end;

procedure TAreasForm.StartStopUpdatingArea;
begin
  UpdateAreaLine;
  UpdateAreaNode(Area);
end;

procedure TAreasForm.CMStartUpdatingArea;
begin
  StartStopUpdatingArea(Msg.InfoPtr);
end;
procedure TAreasForm.CMStopUpdatingArea;
begin
  StartStopUpdatingArea(Msg.InfoPtr);
end;

procedure TAreasForm.AreaOpenedOrClosed;
begin
  UpdateAreaNode(Msg.InfoPtr);
end;

procedure TAreasForm.CMAreaOpened;
begin
  AreaOpenedOrClosed(Msg);
end;
procedure TAreasForm.CMAreaClosed;
begin
  AreaOpenedOrClosed(Msg);
end;

procedure TAreasForm.CMCurrentAreaChanged;

  procedure SetSelectedNode;
  var
    I: Integer;
    Node: TTreeNode;
  begin
    with Areas do
      if Current <> nil then
        with Tree do
          for I := 0 to Pred(Count) do
          begin
            Node := Items[I];
            if Node.Data = Current then
            begin
              Selected := Node;
              Exit;
            end;
          end;
  end;

begin 
  SetSelectedNode;

  UpdateAreaLine;

  IncSearchedStr := '';
end;

procedure TAreasForm.UpdateAreaState;
var
  Flag: Boolean;
begin
  with Area, Node do
    if Valid
    then
    begin
      Flag := UnreadCount <> 0;
      Bold := Flag;
      if Flag
      then ImageIndex := 4
      else ImageIndex := 3;
    end
    else
    begin
      Bold := False;
      if Updating
      then ImageIndex := 2
      else ImageIndex := 1;
    end;
end;

procedure TAreasForm.UpdateAreaNode;

  procedure InvalidateNode(Node: TTreeNode);
  var
    R: TRect;
  begin
    R := TTreeNode(Area.InfoPtr).DisplayRect(False);
    InvalidateRect(Tree.Handle, @R, True);
  end;

var
  Node: TTreeNode;
begin
  Node := Area.InfoPtr;

  UpdateAreaState(Area, Node);
  InvalidateNode(Node);
end;

procedure TAreasForm.TreeChange(Sender: TObject; Node: TTreeNode);
begin
  with Tree, Areas do
    Current := Selected.Data;
end;

procedure TAreasForm.UpdateAreaLine;
begin
  AreaLine.Text := Areas.Current.GetInfo;
{  AreaLine.Update;}
end;

procedure TAreasForm.Navigate;
var
  Node: TTreeNode;
  Area: TArea;
begin
  with Tree do
  begin
    Node := Selected;
    repeat
      if Offset > 0
      then Node := Node.GetNext
      else Node := Node.GetPrev;

      if not Assigned(Node) then
        Break;

      Area := Node.Data;
      if not Assigned(Area) then
        Continue;

      if Area.UnreadCount <> 0 then
      begin
        Selected := Node;
//        Areas.Current := Area;
        Break;
      end;
    until False;
  end;
end;

procedure TAreasForm.AnyNavigationItemClick(Sender: TObject);
var
  Offset: Integer;
begin
  Offset := (Sender as TComponent).Tag;

  Navigate(Offset);
end;

procedure TAreasForm.SetIncSearchedStr;
var
  I: Integer;
  LIncSearchCount: Integer;
begin
  if AIncSearchedStr = '' then
  begin
    if IncSearching then
      Exit; 

{    IncSearchLine.Hide;}
    StatusBar.SimplePanel := False;
    FIncSearchedStr := '';
    Exit;
  end;

  LIncSearchCount := 0;
  with Areas do
    for I := 0 to Pred(Count) do
      with TArea(Items[I]) do
        if Pos(AIncSearchedStr, ID) <> 0 then
          if LIncSearchCount = IncSearchCount
          then
          begin
            FIncSearchedStr := AIncSearchedStr;

            StatusBar.SimpleText := Format(': %s' , [IncSearchedStr]);
            StatusBar.SimplePanel := True;

            IncSearching := True;
            Tree.Selected := InfoPtr;
            IncSearching := False;
            Exit;
          end
          else
            Inc(LIncSearchCount);
  MessageBeep(0);
end;

procedure TAreasForm.TreeKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = SearchAgain
  then
  begin
    Inc(IncSearchCount);
    IncSearchedStr := IncSearchedStr;
  end
  else
  begin
    IncSearchCount := 0;
    if Key = BackSpace
    then IncSearchedStr := Copy(IncSearchedStr, 1, Pred(Length(IncSearchedStr)))
    else IncSearchedStr := IncSearchedStr+UpCase(Key);
  end;
  Key := #0;
end;

procedure TAreasForm.FormStorageRestorePlacement(Sender: TObject);

  function RealCmdShow: Integer;
  var
    StartupInfo: TStartupInfo;
  begin
    GetStartupInfo(StartupInfo);
    Result := StartupInfo.wShowWindow;
  end;

begin
  if RealCmdShow = SW_SHOWMINNOACTIVE then
    ShowWindow(Handle, SW_SHOWMINNOACTIVE);

  SpeedBar.RestoreLayout(FormStorage.IniFile);
  UpdateOptions;
end;

procedure TAreasForm.FormStorageSavePlacement(Sender: TObject);
begin
  SpeedBar.SaveLayout(FormStorage.IniFile);
end;

procedure TAreasForm.UpdateOptions;
begin
  SpeedBarItem.Checked := SpeedBar.Visible;
  ShowHintItem.Checked := SpeedBar.ShowHint;
  StatusLineItem.Checked := StatusBar.Visible;
end;

procedure TAreasForm.ConfigureItemClick(Sender: TObject);
begin
  SpeedBar.Customize(0);
end;

procedure TAreasForm.ShowHintItemClick(Sender: TObject);
begin
  with SpeedBar do
    ShowHint := not ShowHint;

  UpdateOptions;
end;

procedure TAreasForm.HideSpeedBarClick(Sender: TObject);
begin
  SpeedBar.Hide;

  UpdateOptions;
end;

procedure TAreasForm.SpeedBarItemClick(Sender: TObject);
begin
  with SpeedBar do
    Visible := not Visible;

  UpdateOptions;
end;

procedure TAreasForm.StatusLineItemClick(Sender: TObject);
begin
  with StatusBar do
    Visible := not Visible;

  UpdateOptions;
end;

procedure TAreasForm.EnterSelectedAreaItemClick(Sender: TObject);
var
  I: Integer;
  Form: TForm;
  AreaToEnter: TArea;
begin
  IncSearchedStr := ''{   , OnDeactivate   ,
       ,    ,   'Scanning...'};

  if Areas.Count = 0 then
    Exit;

  AreaToEnter := Areas.Current;
  if not Assigned(AreaToEnter) then
    Exit;

  with Screen do
    for I := 0 to Pred(FormCount) do
    begin
      Form := Forms[I];
      if Form is TAreaForm then
        if TAreaForm(Form).Area = AreaToEnter then
        begin
          ActivateWindow(Form.Handle);
          Exit;
        end;
    end;

  ShowHourGlass;
  AreaToEnter.Update;
  TAreaForm.Create(AreaToEnter);
end;

procedure TAreasForm.MinimizeItemClick(Sender: TObject);
begin
  ApplicationMinimize;
end;

procedure TAreasForm.SwitchToTopBrowserClick(Sender: TObject);
begin
  ActivateTopFormOfType(TAreaForm);
end;

procedure TAreasForm.SwitchToTopEditorClick(Sender: TObject);
begin
  ActivateTopFormOfType(TEditorForm);
end;

procedure TAreasForm.FormDeactivate(Sender: TObject);
begin
  IncSearchedStr := '';
end;

{procedure TAreasForm.FirstAreaItemClick(Sender: TObject);
begin
  with Tree do
  begin
    if RowCount = 0 then
      Exit;

    Row := 0;
  end;
end;}

{procedure TAreasForm.LastAreaItemClick(Sender: TObject);
begin
  with Tree do
  begin
    if RowCount = 0 then
      Exit;

    Row := Pred(RowCount);
  end;
end;}

procedure TAreasForm.NetmailItemClick(Sender: TObject);
var
  I: Integer;
begin
  with Areas do
    for I := 0 to Pred(Count) do
      if TArea(Items[I]).ID = 'NETMAIL' then
      begin
        {Tree.Row := I;}
        EnterSelectedAreaItemClick(nil){ };
        Break;
      end;
end;

procedure TAreasForm.FormActivate(Sender: TObject);
begin
  Application.OnHint := ApplicationHint;
end;

procedure TAreasForm.ApplicationHint;
begin
  StatusBar.SimpleText := Application.Hint;
  StatusBar.SimplePanel := StatusBar.SimpleText <> '';
end;

procedure TAreasForm.TreeGetSelectedIndex(Sender: TObject;
  Node: TTreeNode);
begin
  with Node do
    SelectedIndex := ImageIndex;
end;

procedure TAreasForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TAreasForm.UpdateAreasStarterTimer(Sender: TObject);
begin
  UpdateAreasStarter.Enabled := False;
  Areas.Update; //    .processmessages
end;

end.
