unit Common;

interface

uses
  Forms,
  Dialogs,

  MKGlobT;

const
  CRLF = ^M+^J;

const
  BackSpace = #8;
  SearchAgain = ^L;

const
  {$R CURSORS}
  crBuildTree = 1;
  crBuildLetterText = 2;

function PercentOf(Value, MaxValue: Integer): Integer;

function IsMe(const S: string): Boolean;

function BufCRC(var Buffer; BufSize: Word): Longint;

procedure ActivateTopFormOfType(FormClass: TFormClass);

procedure ShowHourGlass;
procedure ShowBuildTree;
procedure ShowBuildLetterText;
procedure HideHourGlass;

function ExtractAddress(S: string; var Address: AddrType): Boolean;

  procedure WordWrap(InSt : string; var OutSt, Overlap : string;
                     Margin : Byte{; PadToMargin : Boolean});
    {-Wrap InSt at Margin, storing the result in OutSt and the remainder
      in Overlap}

function NowWithoutSeconds: TDateTime;

function MessageDlg(const Msg: string; AType: TMsgDlgType;
  AButtons: TMsgDlgButtons; HelpCtx: Longint): Word;

function TimeToStr(Time: TDateTime): string;
function DateToStr(Date: TDateTime): string;
function DateTimeToStr(DateTime: TDateTime): string;

procedure FixKey(var Key: Char);

implementation

uses
  SysUtils,

  WinTypes,
  WinProcs,

  Classes,
  Controls,
  StdCtrls,

  VCLUtils,
  AppUtils,

  CRC32,

  Config;

function PercentOf;
begin
  if MaxValue = 0
  then Result := 0
  else Result := Value * 100 div MaxValue;
end;

function IsMe;
begin
  Result := S = UserName;
end;

function BufCRC;
var
  I: Integer;
begin
  Result := $FFFFFFFF;
  for I := 0 to Pred(BufSize) do
    Result := UpdC32(TByteArray(Buffer)[I], Result);
end;

procedure ActivateTopFormOfType;
var
  Form: TForm;
begin
  Form := FindForm(FormClass);
  if Assigned(Form) then
    ActivateWindow(Form.Handle);
end;

procedure SetScreenCursor(Cursor: TCursor);
begin
  Screen.Cursor := Cursor;
end;

procedure ShowHourGlass;
begin
  SetScreenCursor(crHourglass);
end;

procedure HideHourGlass;
begin
  SetScreenCursor(crDefault);
end;

procedure ShowBuildTree;
begin
  SetScreenCursor(crBuildTree);
end;

procedure ShowBuildLetterText;
begin
  SetScreenCursor(crBuildLetterText);
end;

function ExtractAddress;
var
  P, B: Byte;
begin
  P := 0;
  for B := Length(S) downto 1 do
    if S[B] = '(' then
    begin
      P := B;
      Break;
    end;

  if P <> 0 then
  begin
    Delete(S, 1, P);
    P := Pos(' ', S);  if P <> 0 then Delete(S, 1, P);
  end;
  P := Pos(')', S);  if P <> 0 then Delete(S, P, $FF);
  Result := ParseAddr(S, MainAddress, Address);
end;

  procedure WordWrap;
  var
    InStLen : Byte;
//    OutStLen : Byte;
//    OvrLen : Byte;
    EOS, BOS : Word;
  begin
    InStLen := Length(InSt);

    {find the end of the output string}
    if InStLen > Margin then begin
      {find the end of the word at the margin, if any}
      EOS := Margin;
      while (EOS <= InStLen) and (InSt[EOS] <> ' ') do
        Inc(EOS);
      if EOS > InStLen then
        EOS := InStLen;

      {trim trailing blanks}
      while (InSt[EOS] = ' ') and (EOS > 0) do
        Dec(EOS);

      if EOS > Margin then begin
        {look for the space before the current word}
        while (EOS > 0) and (InSt[EOS] <> ' ') do
          Dec(EOS);

        {if EOS = 0 then we can't wrap it}
        if EOS = 0 then
          EOS := Margin
        else
          {trim trailing blanks}
          while (InSt[EOS] = ' ') and (EOS > 0) do
            Dec(EOS);
      end;
    end
    else
      EOS := InStLen;

    {copy the unwrapped portion of the line}
    OutSt := Copy(InSt, 1, EOS); //    OutStLen := EOS;Move(InSt[1], OutSt[1], OutStLen);

    {find the start of the next word in the line}
    BOS := EOS+1;
    while (BOS <= InStLen) and (InSt[BOS] = ' ') do
      Inc(BOS);

    if BOS > InStLen then
      Overlap := '' //OvrLen := 0
    else begin
      {copy from the start of the next word to the end of the line}
      Overlap := Copy(InSt, BOS, Succ(InStLen-BOS)); // OvrLen := Succ(InStLen-BOS); Move(InSt[BOS], Overlap[1], OvrLen);
    end;
(*
    {pad the end of the output string if requested}
    if PadToMargin and (OutStLen < Margin) then begin
      FillChar(OutSt[OutStLen+1], Margin-OutStLen, ' ');
      OutStLen := Margin;
    end;*)
  end;

function NowWithoutSeconds;
var
  DateTime: TDateTime;
  Hour, Min, Sec, MSec: Word;
begin
  DateTime := Now;
  DecodeTime(DateTime, Hour, Min, Sec, MSec);
  Result := Trunc(DateTime)+EncodeTime(Hour, Min, 0{!}, 0{!});
end;

function MessageDlg;
var
  Btn: TButton;
  SavedCursorPos: TPoint;
begin
  Result := 0;

  GetCursorPos(SavedCursorPos);

  if HelpCtx <> 0 then
    Include(AButtons, mbHelp){ -> Borland!};
  with CreateMessageDialog(Msg, AType, AButtons) do
  try
    HelpContext := HelpCtx;
{    if X > -1 then W.Left := X;
    if Y > -1 then W.Top := Y;}
    ScaleBy(Screen.PixelsPerInch, 96);

    Btn := FindComponent('YES') as TButton;
    if not Assigned(Btn) then
      Btn := FindComponent('OK') as TButton;
    if Assigned(Btn){     } then
      with Btn, ClientToScreen(Point(0, 0)) do
        SetCursorPos(X+Width div 2, Y+Height div 2);
    Result := ShowModal;
  finally
    Free;

    with SavedCursorPos do
      SetCursorPos(X, Y);
  end;
end;

{$I SysUtils.INC}

function TimeToStr;
//var
//  Hour, Min, Sec, MSec: Word;
begin
//  DecodeTime(Time,  Hour, Min, Sec, MSec);
//  Result := Format('%d:%2d', [Hour, Min]);
//  DecodeTime(Time,  Hour, Min, Sec, MSec);
  Result := FormatDateTime('h:nn', Time);
end;

function DateToStr;
var
  Year, Month, Day: Word;
begin
  DecodeDate(Date,  Year, Month, Day);
  Result :=
    Format('%s, %d %s %d', [
      LoadStr(SLongDayNames+Pred(DayOfWeek(Date))),
      Day,
      LoadStr(SLongMonthNames+Pred(Month)),
      Year mod 100]);
end;

function DateTimeToStr;
begin
  Result := Format('%s  %s', [DateToStr(DateTime), TimeToStr(DateTime)]);
end;

procedure LoadCursors;
begin
  Screen.Cursors[crBuildTree] := LoadCursor(HInstance, 'BUILDTREE');
  Screen.Cursors[crBuildLetterText] := LoadCursor(HInstance, 'BUILDLETTERTEXT');
end;

procedure FixKey;
begin
  if Key = ''{} then
    Key := 'H'{};
end;

initialization
  LoadCursors;
end.
