unit Config;

interface

uses
  Classes,

  MKGlobT;

const
  UserSection = 'User';
  AKAsSection = 'AKAs';
  NewLetterTemplateSection = 'NewLetterTemplate';
  QuoteReplyTemplateSection = 'QuoteReplyTemplate';
  LineIndent = 'Line';

var
  UserName: string;
  MainAddress: AddrType;
  AKAs,
  NewLetterTemplate,
  QuoteReplyTemplate: TStrings;

const
  NameIndent = 'Name';
  MainAddressIndent = 'MainAddress';

procedure ReadConfig;
procedure WriteConfig;

implementation

uses
  SysUtils,

  Registry,

  AppUtils, 

  Common;

const
  EmptyStringChar = '-';

procedure ReadConfig;
var
  RegIniFile: TRegIniFile;

  procedure ReadMemo(const SectionName: string; List: TStrings);

    function Unprepare(List: TStrings): TStrings;

      function DoUnprepare(const S: string): string;
      begin
        Result := Copy(S, Succ(Pos('=', S)), $FF);
        if Result = EmptyStringChar then
          Result := '';
      end;

    var
      I: Integer;
    begin
      Result := List;

      with List do
        for I := 0 to Pred(Count) do
          Strings[I] := DoUnprepare(Strings[I]);
    end;

  begin
    if RegIniFile.ReadString(SectionName, LineIndent+'0', '') <> ''{-  ?} then
    begin
      List.Clear;
      RegIniFile.ReadSectionValues(SectionName, List);
      Unprepare(List);
    end;
  end;

begin
  RegIniFile := TRegIniFile.Create(GetDefaultIniName);
  with RegIniFile do
    try
      UserName := ReadString(UserSection, NameIndent, 'Alexander Petrosyan');
      ExtractAddress(ReadString(UserSection, MainAddressIndent, '2:5020/468.8'), MainAddress);

      ReadMemo(AKAsSection, AKAs);
      ReadMemo(NewLetterTemplateSection, NewLetterTemplate);
      ReadMemo(QuoteReplyTemplateSection, QuoteReplyTemplate);
    finally
      Free;
    end;
end;

procedure WriteConfig;
var
  RegIniFile: TRegIniFile;

  procedure WriteMemo(const SectionName: string; List: TStrings);

    procedure WriteSection(const SectionName: string; List: TStrings);

      function NotEmpty(const S: string): string;
      begin
        Result := S;
        if Result = '' then
          Result := EmptyStringChar;
      end;

    var
      I: Integer;
    begin
      with List do
        for I := 0 to Pred(Count) do
          RegIniFile.WriteString(SectionName, Format(LineIndent+'%d', [I]), NotEmpty(Strings[I]));
    end;

  begin
    RegIniFile.EraseSection(SectionName);
    WriteSection(SectionName, List);
  end;

begin
  RegIniFile := TRegIniFile.Create(GetDefaultIniName);
  with RegIniFile do
    try
      WriteString(UserSection, NameIndent, UserName);
      WriteString(UserSection, MainAddressIndent, AddrStr(MainAddress));

      WriteMemo(AKAsSection, AKAs);
      WriteMemo(NewLetterTemplateSection, NewLetterTemplate);
      WriteMemo(QuoteReplyTemplateSection, QuoteReplyTemplate);
    finally
      Free;
    end;
end;

procedure CleanUp;
begin
  AKAs.Free;
  NewLetterTemplate.Free;
  QuoteReplyTemplate.Free;
end;

begin
  AKAs := TStringList.Create;
  NewLetterTemplate := TStringList.Create;
  QuoteReplyTemplate := TStringList.Create;

  with NewLetterTemplate do
  begin
    Add(', @TFName!');
    Add('');
    Add('@Position');
    Add('');
    Add(' , .');
    Add('');
  end;

  with QuoteReplyTemplate do
  begin
    Add('@ODate  @OName(@OAddr) > @DName:');
    Add('@Position');
    Add('@Quote');
    Add('');
    Add(' , .');
    Add('');
  end;

  AddExitProc(CleanUp);
end.
