unit EditorF;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, ExtCtrls, SpeedBar, StdCtrls, Placemnt, ComCtrls,

  Radio,
  Commands,
  Mail,
  ReFormat,

  AreaF, IPopup;

type
  ECantWriteMessage = Exception;

  TEditorAction = (
    eaNew, eaEdit, eaReply, eaComment, eaReplyInAnotherArea, eaCommentInAnotherArea);

type
  TEditorForm = class(TForm)
    SpeedBar: TSpeedBar;
    MainMenu: TMainMenu;
    LetterItem: TMenuItem;
    SaveItem: TMenuItem;
    ExitItem: TMenuItem;
    HelpItem: TMenuItem;
    AboutItem: TMenuItem;
    N4: TMenuItem;
    HowToUseHelpItem: TMenuItem;
    SearchforHelpOnItem: TMenuItem;
    ContentsItem: TMenuItem;
    SwitchItem: TMenuItem;
    SpeedBarItem: TMenuItem;
    StatusLineItem: TMenuItem;
    SpeedBarPopupMenu: TPopupMenu;
    ConfigureItem: TMenuItem;
    ShowHintItem: TMenuItem;
    HideSpeedBar: TMenuItem;
    SpeedBarHelpItem: TMenuItem;
    FormStorage: TFormStorage;
    SaveSpeedItem: TSpeedItem;
    HiddenEditorPopupMenu: TPopupMenu;
    HeaderItem: TMenuItem;
    Bevel1: TBevel;
    HiddenHeaderPopupMenu: TPopupMenu;
    NextControlItem: TMenuItem;
    WindowItem: TMenuItem;
    WindowListItem: TMenuItem;
    N1: TMenuItem;
    SwitchToAreasItem: TMenuItem;
    SwitchToAreaItem: TMenuItem;
    NextControlItem2: TMenuItem;
    PrevControlItem: TMenuItem;
    TabItem: TMenuItem;
    TabItem2: TMenuItem;
    OptionsItem: TMenuItem;
    UserItem: TMenuItem;
    N2: TMenuItem;
    Panel3: TPanel;
    HeaderPanel: TPanel;
    ClientToPanel: TPanel;
    ToEdit: TEdit;
    RightToPanel: TPanel;
    Label2: TLabel;
    ToAddressEdit: TEdit;
    SubjectPanel: TPanel;
    SubjectEdit: TEdit;
    FromPanel: TPanel;
    ClientFromPanel: TPanel;
    FromEdit: TEdit;
    RightFromPanel: TPanel;
    DateTimeWritten: TLabel;
    FromAddressEdit: TEdit;
    RichEdit: TRichEdit;
    Panel4: TPanel;
    StatusBar: TStatusBar;
    EditItem: TMenuItem;
    GoToItem: TMenuItem;
    ReplaceItem: TMenuItem;
    FindItem: TMenuItem;
    N5: TMenuItem;
    PasteItem: TMenuItem;
    CopyItem: TMenuItem;
    CutItem: TMenuItem;
    N6: TMenuItem;
    UndoItem: TMenuItem;
    CutSpeedItem: TSpeedItem;
    CopySpeedItem: TSpeedItem;
    ExitSpeedItem: TSpeedItem;
    PasteSpeedItem: TSpeedItem;
    UndoSpeedItem: TSpeedItem;
    DelItem: TMenuItem;
    EditIndirectPopupMenu: TIndirectPopupMenu;
    LetterIndirectPopupMenu: TIndirectPopupMenu;
    procedure FormCreate(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure FormStorageSavePlacement(Sender: TObject);
    procedure FormStorageRestorePlacement(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure SpeedBarItemClick(Sender: TObject);
    procedure StatusLineItemClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure HeaderItemClick(Sender: TObject);
    procedure ConfigureItemClick(Sender: TObject);
    procedure ShowHintItemClick(Sender: TObject);
    procedure HideSpeedBarClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SubjectEditChange(Sender: TObject);
    procedure RichEditChange(Sender: TObject);
    procedure AnyEditChange(Sender: TObject);
    procedure RichEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure NextControlItemClick(Sender: TObject);
    procedure SwitchToAreasItemClick(Sender: TObject);
    procedure SwitchToAreaItemClick(Sender: TObject);
    procedure TabItemClick(Sender: TObject);
    procedure RichEditEnter(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure RichEditSelectionChange(Sender: TObject);
    procedure UndoItemClick(Sender: TObject);
    procedure CutItemClick(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure PasteItemClick(Sender: TObject);
    procedure DelItemClick(Sender: TObject);
    procedure SubjectEditKeyPress(Sender: TObject; var Key: Char);
  private
    procedure CreateParams(var Params: TCreateParams); override;
  private
    XYLine, ModifiedFlagLine, LetterLine, AreaLine: TStatusPanel;
  private
    function CloseQuery: Boolean;
    procedure UpdateAreaLine;
    procedure CMAreasChanged(var Msg: TRadioMessage);
      message CM_AreasChanged;
    procedure CMAreaDontWantToClose(var Msg: TRadioMessage);
      message CM_AreaDontWantToClose;
    procedure CMAreaClosed(var Msg: TRadioMessage);
      message CM_AreaClosed;
  private
    { Private declarations }
    procedure UpdateOptions;
    procedure Save;
  private
    Action: TEditorAction;
    AreaForm: TAreaForm;
    SourceLetter: TLetter;
    SourceLetterID: Longint;
    SourceLetterFormattedStrings: TFormattedStrings;
  private
    FModified: Boolean;
    procedure SetModified(AModified: Boolean);
    property Modified: Boolean read FModified write SetModified;
  private
    LetterFilled: Boolean;
  private
    procedure SaveSourceLetter;
    function RestoreSourceLetter: Boolean;
  private
    procedure ApplicationHint(Sender: TObject);
  private
    procedure NoUndo;
  public
    { Public declarations }
    Area: TArea;

    constructor Create(AArea: TArea; AAreaForm: TAreaForm; AAction: TEditorAction);
  end;

implementation

{$R *.DFM}
{$R *}

uses
  VCLUtils,
  StrUtils,

  MKGlobT,
  MKString,
  MKMsgAbs,

  AreasF,

  Common,
  Config,
  TextKind,
  Tokens;

{$I Version.INC}

const
  WrapCharWidth = 80-1{ ,   Ged'  ScrollBar};

procedure TEditorForm.CreateParams;
begin
  inherited CreateParams(Params);
  Params.WndParent := 0
end;

constructor TEditorForm.Create;
begin
  Area := AArea;
  AreaForm := AAreaForm;
  Action := AAction;

  if Action <> eaNew then
    SourceLetter := Area.Current;

  inherited Create(Application);
end;

procedure TEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TEditorForm.FormShow(Sender: TObject);
begin
  if Action = eaEdit then
    RichEdit.SetFocus;

  SubjectEdit.SetFocus;

{  case Action of
    eaNew,
    eaEdit:
      SubjectEdit.SetFocus;
    eaReply,
    eaComment:
      RichEdit.SetFocus;
  end;}
end;

procedure TEditorForm.NextControlItemClick(Sender: TObject);
begin
  SendMessage(Handle, WM_NEXTDLGCTL, (Sender as TMenuItem).Tag, 0);
end;

procedure TEditorForm.SetModified;
begin
  FModified := AModified;

  with ModifiedFlagLine do
    if AModified
    then Text := ''
    else Text := '';
end;

procedure TEditorForm.FormCreate(Sender: TObject);

  function GetSourceLetterFormattedStrings: TFormattedStrings;
  begin
    Result := TFormattedStrings.Create;

    if not Assigned(SourceLetter) then
      Exit;

    with Area, Disk^, SourceLetter do
    begin
      Seek;
      MsgStartUp;
      ReadText(Result);

      Result.AnalizeAndReFormat(
        False{IncludeKludges},
        WrapCharWidth,
        0, nil,
        Disk);
    end;
  end;

  procedure FillHeader;
  begin
    with SourceLetter, SourceLetterFormattedStrings do
    begin
      case Action of
        eaNew,
        eaReply,
        eaComment:
        begin
          FromEdit.Text := UserName;
          FromAddressEdit.Text := AddrStr(MainAddress);
        end;
        eaEdit:
        begin
          FromEdit.Text := From;
          FromAddressEdit.Text := AddrStr(FromAddr);
        end;
      end;
      with ToEdit do
        case Action of
          eaNew: Text := '';
          eaEdit: Text := WhoTo;
          eaReply: Text := From;
          eaComment: Text := WhoTo;
        end;
      with SubjectEdit do
        case Action of
          eaNew:
            Text := '';
          eaEdit,
          eaReply,
          eaComment:
            Text := Subject;
        end;

      DateTimeWritten.Caption := DateTimeToStr(Now);

      {ToAddressEdit.Text := AddrStr(FromAddr);     }
    end;
  end;

begin
  Icon.Handle := LoadIcon(hInstance, 'Z_EDITORF_ICON');

  UserItem.OnClick := AreasForm.UserItemClick;
  WindowListItem.OnClick := AreasForm.WindowListItemClick;
  AboutItem.OnClick := AreasForm.AboutItemClick;

  XYLine := StatusBar.Panels[0];
  ModifiedFlagLine := StatusBar.Panels[1];
  LetterLine := StatusBar.Panels[2];
  AreaLine := StatusBar.Panels[3];

  SourceLetterFormattedStrings := GetSourceLetterFormattedStrings;

  FillHeader;
  RichEdit.Clear;
  RichEdit.DefAttributes.Color := tkColors[tkNormal];

  XYLine.Text := '';
  ModifiedFlagLine.Text := '';
  LetterLine.Text := '';
  UpdateAreaLine;

  SaveSourceLetter;

  Modified := False;
end;

procedure TEditorForm.UpdateAreaLine;
begin
  with Area do
    AreaLine.Text := Format('%s(%s)', [Comment, ID]);
end;

procedure TEditorForm.SaveSourceLetter;
begin
  if Assigned(SourceLetter) then
  begin
    SourceLetterID := SourceLetter.ID;
    SourceLetter := nil;
  end;
end;

function TEditorForm.RestoreSourceLetter;
begin
  if SourceLetterID <> 0 then
    SourceLetter := Area.LetterByID(SourceLetterID);

  Result := Assigned(SourceLetter);
end;

procedure TEditorForm.FormStorageRestorePlacement(Sender: TObject);
begin
  SpeedBar.RestoreLayout(FormStorage.IniFile);

  UpdateOptions;
end;

procedure TEditorForm.FormStorageSavePlacement(Sender: TObject);
begin
  SpeedBar.SaveLayout(FormStorage.IniFile);
end;

procedure TEditorForm.SaveItemClick(Sender: TObject);
begin
  Save;
  Close;
end;

procedure TEditorForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TEditorForm.UpdateOptions;
begin
  SpeedBarItem.Checked := SpeedBar.Visible;
  ShowHintItem.Checked := SpeedBar.ShowHint;
  StatusLineItem.Checked := StatusBar.Visible;
end;

procedure TEditorForm.SpeedBarItemClick(Sender: TObject);
begin
  with SpeedBar do
    Visible := not Visible;

  UpdateOptions;
end;

procedure TEditorForm.StatusLineItemClick(Sender: TObject);
begin
  with StatusBar do
    Visible := not Visible;

  UpdateOptions;
end;

procedure TEditorForm.ConfigureItemClick(Sender: TObject);
begin
  SpeedBar.Customize(0);
end;

procedure TEditorForm.ShowHintItemClick(Sender: TObject);
begin
  with SpeedBar do
    ShowHint := not ShowHint;

  UpdateOptions;
end;

procedure TEditorForm.HideSpeedBarClick(Sender: TObject);
begin
  SpeedBar.Hide;

  UpdateOptions;
end;

function TEditorForm.CloseQuery;
begin
  Result := True;

  ActivateWindow(Handle);

  if Modified then
    case MessageDlg(' ?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes: Save;
      mrNo: Modified := False;
      mrCancel: Result := False;
    end;
end;

procedure TEditorForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := CloseQuery;
end;

procedure TEditorForm.HeaderItemClick(Sender: TObject);
begin
  SubjectEdit{FromEdit}.SetFocus;
end;

procedure TEditorForm.CMAreaDontWantToClose;
begin
  if Msg.InfoPtr = Area then
    LongBool(Msg.Result) := not CloseQuery;
end;

procedure TEditorForm.CMAreaClosed;
begin
  if Msg.InfoPtr = Area then
    Close;
end;

procedure TEditorForm.SubjectEditChange(Sender: TObject);
begin
  with Area do
    Caption := SubjectEdit.Text;
//    Caption := Format('%s  [%s - %s]', [SubjectEdit.Text, Comment, ID]);
  AnyEditChange(nil);
end;

procedure TEditorForm.RichEditChange(Sender: TObject);
begin
  LetterLine.Text := Format('%d ', [StrLen(RichEdit.Lines.GetText)]);

  Modified := True;
end;

procedure TEditorForm.AnyEditChange(Sender: TObject);
begin
  Modified := True;
end;

procedure TEditorForm.RichEditKeyPress(Sender: TObject; var Key: Char);
var
  LineStart, LineNum: Integer;
  S: string;
  X: Integer;
  AtHome, AtEnd: Boolean;
  P: Integer;
begin
  FixKey(Key);

  if Key = ^M then
    with RichEdit do
    begin
      LineNum := Perform(EM_LINEFROMCHAR, SelStart, 0);
      LineStart := Perform(EM_LINEINDEX, LineNum, 0);
      S := Lines[LineNum];
      P := QuotePos(S);
      if P >= 1 then  //    quoting
      try
        Lines.BeginUpdate;

        X := SelStart-LineStart;
        AtHome := X = 0;
        AtEnd := X = Succ( Pred(Length(S)) );
        if AtHome or AtEnd
        then  //   quoting' -  
        begin
          SelText := CRLF+CRLF;
          SelStart := SelStart - Length(CRLF+CRLF);
          SelLength := Length(CRLF+CRLF);
          SelAttributes.Color := tkColors[tkNormal];
          SelLength := 0;
          if AtEnd then
            SelStart := SelStart + Length(CRLF);
        end
        else  //   quoting'    -   chars>
        begin
          while
            (Succ(P) <= Length(S)) and
            (S[Succ(P)] = ' ') do
            Inc(P);
          SelText := CRLF+Copy(S, 1, P);
        end;
        Key := #0;
        NoUndo;
      finally
        Lines.EndUpdate;
      end;
    end;
end;

procedure TEditorForm.Save;

  function CreateLetter: TLetter;
  begin
    Result := TLetter.Create(Area,
      -1{  }, 0,
      SubjectEdit.Text, FromEdit.Text, ToEdit.Text,
      NowWithoutSeconds,
      False);

    with Result do
    begin
      if Assigned(SourceLetter) then
        case Action of
          eaReply,
          eaComment:
            Reference := SourceLetter.Number;
          eaEdit:
            Reference := SourceLetter.Reference;
        end;
      JustRead := True;
    end;
  end;

  procedure Process(Letter: TLetter);

    procedure DoSave;

      function UniqueLong: Longint;
      type
        DLongRec = record
          Lo, Hi: Longint;
        end;
      var
        DateTime: TDateTime;
      begin
        DateTime := Now;
        Result := DLongRec(DateTime).Lo xor DLongRec(DateTime).Hi + Random($FFFF);
      end;

    var
      I: Integer;
      S: string;
      WriteError: Integer;
      FromAddr: AddrType;
    begin
      with Area.Disk^, Letter, SourceLetterFormattedStrings do
      begin
        StartNewMsg;

        ParseAddr(FromAddressEdit.Text, MainAddress, FromAddr);

        SetFrom(From);
        SetTo(WhoTo);
        SetSubj(Subject);
        {SetPriv(Priv);}
        SetDateTime(DateTime);
        SetOrig(FromAddr);
        SetLocal(True);{?}
        {SetEcho(True);{?}
        {SetDest(DestAddr);}
        SetMailType(mmtEchoMail);

        if Reference <> 0 then
          SetRefer(Reference);
        SetRead(True);

        if Action = eaEdit
        then
        begin
          DoKludgeLn(Format(^A'MSGID: %s', [MsgID]));
          DoKludgeLn(Format(^A'REPLY: %s', [Reply]));
        end
        else
        begin
          DoKludgeLn(Format(^A'MSGID: %s %8x', [AddrStr(FromAddr), UniqueLong]));
          if MsgID <> '' then
            DoKludgeLn(Format(^A'REPLY: %s', [MsgID]));
        end;

        with TFormattedStrings.Create do
          try
            Assign(RichEdit.Lines);

            AnalizeAndReFormat(
              False{IncludeKludges},
              WrapCharWidth,
              0, nil,
              nil);

            for I := 0 to Pred(Count) do
            begin
              S := Strings[I];
              if (S <> '') and (S[1] = ^A)
              then DoKludgeLn(S)
              else
                if I = Pred(Count)
                then DoString(S)
                else DoStringLn(S);
            end;
          finally
            Free;
          end;

        WriteError := WriteMsg;
        if WriteError <> 0 then
          raise ECantWriteMessage.CreateFmt('   .  #%d',
            [WriteError]);

        Number := GetMsgNum;

        if Action = eaEdit{SourceLetter  ,    nil} then
          SourceLetter.Delete;
      end;
    end;

  begin
    try
      DoSave;

      with Area do
      begin
        if Action = eaEdit
        then
        begin
          SourceLetter.CopyFrom(Letter);
          VisualChange(SourceLetter);
          Letter.Free;
        end
        else
          VisualAdd(Letter);
      end;
    except
      Letter.Free{ - , .    ?
          @  __   ?
          ....};
    end;
  end;

begin
  if not RestoreSourceLetter and (Action = eaEdit) then
  begin
    Modified := False{        };
    { ,   , }
    MessageDlg('  ,     .'^M+
      '  .',
      mtError, [mbOK], 0);
    Close;
    Exit;
  end;

  Process(CreateLetter);
  Modified := False;
end;

procedure TEditorForm.SwitchToAreasItemClick(Sender: TObject);
begin
  if Assigned(AreasForm)
  then ActivateWindow(AreasForm.Handle)
  else AreasForm := TAreasForm.Create(Application);
end;

procedure TEditorForm.SwitchToAreaItemClick(Sender: TObject);
begin
  ActivateWindow(AreaForm.Handle);
end;

procedure TEditorForm.TabItemClick(Sender: TObject);
begin
  RichEdit.SetSelTextBuf('  ');
end;

procedure TEditorForm.RichEditEnter(Sender: TObject);

  procedure Fill;

    procedure DoFill;

      function GetTokenValues: TStrings;
      var
        I: Integer;
      begin
        Result := TStringList.Create;

        with Result do
        begin
          Add(AddrStr(MainAddress))        {Current user address};
          Add(ExtractWord(UserName, 1))    {Current user first name};
          Add(ExtractWord(UserName, 2))    {Current user last name};
          Add(UserName)                    {Current user name};
          Add(DateToStr(Now))              {Current date};
          Add(TimeToStr(Time))             {Current time};
          Add(Area.Comment)                {Current area description};
          Add(Area.ID)                     {Current echoid};

          if Action = eaNew
          then
            for I := 1 to 8+4 do
              Add(''){   ? ׸  }
          else
            with SourceLetter do
            begin
              Add(AddrStr(SourceLetterFormattedStrings.FromAddr)){Original address};
              Add(ExtractWord(From, 1))        {Original first name};
              Add(ExtractWord(From, 2))        {Original last name};
              Add(From)                        {Original name};
              Add(DateToStr(DateTime))         {Original date};
              Add(TimeToStr(DateTime))         {Original time};
              Add(Area.Comment)                {Original area description if moved; else current};
              Add(Area.ID)                     {Original echoid if moved; otherwise current};

              Add(AddrStr(SourceLetterFormattedStrings.ToAddr)){Destination address};
              Add(ExtractWord(WhoTo, 1))       {Destination first name};
              Add(ExtractWord(WhoTo, 2))       {Destination last name};
              Add(WhoTo)                       {Destination name};
            end;

          Add(ToAddressEdit.Text)          {Destination to address};
          Add(ExtractWord(ToEdit.Text, 1)) {Destination to first name};
          Add(ExtractWord(ToEdit.Text, 2)) {Destination to last name};
          Add(ToEdit.Text)                 {Destination to name};
        end;
      end;

      procedure FillByTemplate(TokenValues: TStrings);

        function DoTemplate(Template: TStrings): TFormattedStrings;

          procedure Quote;

            function GetInitials(const S: string): string;

              function FirstChar(const S: string): string;
              begin
                if S = ''
                then Result := ''
                else Result := S[1];
              end;

            begin
              Result :=
                FirstChar(ExtractWord(S, 1))+
                FirstChar(ExtractWord(S, 2))+
                FirstChar(ExtractWord(S, 3));
            end;

          var
            QuoteInitials: string;
            I: Integer;
            S: string;
          begin
            with SourceLetterFormattedStrings do
            begin
              QuoteInitials := GetInitials(TokenValues[11{OName}])+'> ';

              for I := 0 to Pred(Count) do
              begin
                S := Strings[I];
                case GetTextKind(S) of
                  tkNormal,
                  tkKludge: if S <> '' then S := QuoteInitials+S;
                  tkQuote:
                  begin
                    S := DelBSpace(S);
                    System.Insert('>', S, Pos('>', S));
                  end;
                  tkTearline,
                  tkOrigin: Continue{ };
                end;
                Result.Add(S);
              end;
            end;
          end;

          function GetTokenNum(S: string): Integer;
          var
            I: Integer;
          begin
            Result := -1;
            S := UpperCase(S);
            for I := 0 to High(TokenNames) do
              if S = UpperCase(StrPas(TokenNames[I])) then
              begin
                Result := I;
                Break;
              end;
          end;

        var
          I: Integer;
          S: string;
          P: Integer;
          W: Integer;
          TokenNum: Integer;
        begin
          Result := TFormattedStrings.Create;

          with Template do
            for I := 0 to Pred(Count) do
            begin
              S := Strings[I];
              if S = TokenStartChar+StrPas(TokenNames[-2]){Position}
              then
                {!z}
              else
                if S = TokenStartChar+StrPas(TokenNames[-1]){Quote}
                then
                  Quote
                else
                begin
                  repeat
                    P := Pos(TokenStartChar, S);
                    if P = 0 then
                      Break;

                    System.Delete(S, P, 1);
                    for W := MinTokenWidth to MaxTokenWidth do
                    begin
                      TokenNum := GetTokenNum(Copy(S, P, W));
                      if TokenNum >= 0 then
                      begin
                        System.Delete(S, P, W);
                        System.Insert(TokenValues[TokenNum], S, P);
                        Break;
                      end;
                    end;
                  until False;
                  Result.Add(S);
                end;
            end;

          Result.AnalizeAndReFormat(
            False{IncludeKludges},
            WrapCharWidth,
            0, nil,
            nil);
        end;

        procedure SetRichEditLinesTo(Strings: TStrings);
        var
          Line: Integer;
          S: string;
        begin
          RichEdit.SetTextBuf(PChar(Strings.Text));
          with Strings do
            for Line := 0 to Pred(Count) do
              with RichEdit do
              begin
                S := Strings[Line];
                SelStart := Perform(EM_LINEINDEX, Line, 0);
                SelLength := Length(S){+Length(CRLF)};
                SelAttributes.Color := tkColors[GetTextKind(S)];
              end;
        end;

      var
        Template: TStrings;
        MessageBody: TStrings;
      begin
        case Action of
          eaEdit:
          begin
            RichEdit.Lines := SourceLetterFormattedStrings;
            Exit;
          end;
          eaNew:
            Template := NewLetterTemplate;
          eaReply,
          eaComment:
            Template := QuoteReplyTemplate;
          else
            Template := nil { ...};
        end;

        MessageBody := DoTemplate(Template);
        with MessageBody do
        begin
          Add('---  '+Version);
          Add(Format(' * Origin: %s (%s)', [
            'Goodwin->Wolfe: H , .   ',
            AddrStr(MainAddress)]));
        end;
        SetRichEditLinesTo(MessageBody);
      end;

    var
      TokenValues: TStrings;
    begin
      TokenValues := nil{ ,   ...};
      if Action <> eaEdit then
        TokenValues := GetTokenValues;

      FillByTemplate(TokenValues);
    end;

  begin
    with RichEdit, Lines do
    begin
      BeginUpdate;
      try
        DoFill;

        {  }
        SelStart := Length(Text) - Length(CRLF);
        SelLength := Length(CRLF);
        SelText := '';

        SelStart := 0;

        NoUndo;
      finally
        EndUpdate;
      end;
    end;
    Modified := False;
  end;

begin
  if LetterFilled then
    Exit;
  LetterFilled := True;

  ShowBuildLetterText;
  try
    RestoreSourceLetter;
    Fill;
    SaveSourceLetter;
  finally
    HideHourGlass;
  end;
end;

procedure TEditorForm.FormDestroy(Sender: TObject);
begin
  SourceLetterFormattedStrings.Free;
end;

procedure TEditorForm.FormActivate(Sender: TObject);
begin
  Application.OnHint := ApplicationHint;
end;

procedure TEditorForm.ApplicationHint;
begin
  StatusBar.SimpleText := Application.Hint;
  StatusBar.SimplePanel := StatusBar.SimpleText <> '';
end;

procedure TEditorForm.RichEditSelectionChange(Sender: TObject);
var
  LineStart, Y: Integer;
begin
  with RichEdit do
  begin
    Y := Perform(em_LineFromChar, SelStart, 0);
    LineStart := Perform(em_LineIndex, Y, 0);
    XYLine.Text := Format('%3d:%3d', [Succ(Y), Succ(SelStart-LineStart)]);
  end;
end;

procedure TEditorForm.CMAreasChanged;
begin
  UpdateAreaLine;
end;

procedure TEditorForm.UndoItemClick(Sender: TObject);
begin
  with RichEdit do
    if HandleAllocated then SendMessage(Handle, EM_UNDO, 0, 0);
end;

procedure TEditorForm.CutItemClick(Sender: TObject);
begin
  RichEdit.CutToClipboard;
end;

procedure TEditorForm.CopyItemClick(Sender: TObject);
begin
  RichEdit.CopyToClipboard;
end;

procedure TEditorForm.PasteItemClick(Sender: TObject);
begin
  RichEdit.PasteFromClipboard;
end;

procedure TEditorForm.DelItemClick(Sender: TObject);
begin
  RichEdit.SetSelTextBuf(nil);
end;

procedure TEditorForm.SubjectEditKeyPress(Sender: TObject; var Key: Char);
begin
  FixKey(Key);
end;

procedure TEditorForm.NoUndo;
begin
  Perform(EM_EMPTYUNDOBUFFER, 0, 0);
end;

end.
