unit Mail;

interface

uses winprocs,
  Messages,
  SysUtils,

  Classes,

  MKGlobT,
  MKMsgAbs;

type
  ECantFindAreasBBS = class(Exception);
  ECantOpenArea = class(Exception);
  ECantCloseArea = class(Exception);
  EArea_GetCurrent = class(Exception);
  ELetterNotFound = class(Exception);

type
  TAreaBase = class;
  TArea = class;
  TFilteredLetterList = class;
  TLetter = class;

  TBaseList = class(TList)
  protected
    class function Compare(PA, PB: Pointer): Integer; virtual; abstract;
  private
    procedure QuickSort(L, R: Integer);
  public
    procedure Sort;
    procedure FreeAll;
  end;

  TAreasBase = class(TBaseList)
  public
    destructor Destroy; override;
  end;

  TAreasFromCFG = class(TAreasBase)
  protected
    class function Compare(PA, PB: Pointer): Integer; override;
  public
    constructor LoadFromCFG;
  end;

  TAreas = class(TAreasFromCFG)
  protected
    class function Compare(PA, PB: Pointer): Integer; override;
  private
    FRaw: TList;
    FValid: Boolean;
    FCurrent: TArea;

    procedure CMCurrentAreaChanged;
    procedure SetCurrent(ACurrent: TArea);
  public
    Updating: Boolean;
    property Current: TArea read FCurrent write SetCurrent;

    constructor Create;
    destructor Destroy; override;

    procedure Invalidate;
    procedure VisibleSort;
    procedure Update;

    function GetInfo: string;
    function GetTotalUnreadCount: Longint;
  end;

  TLetterList = class(TBaseList)
    function LetterByNumber(Number: Integer): TLetter;
  end;

  TAreaBase = class(TLetterList)
  public
    ID, Comment, MKID: string;
    constructor Create(const AID, AComment, AMKID: string);
    destructor Destroy; override;

    procedure CopyFrom(Source: TAreaBase);
  end;

  TAreaWithDisk = class(TAreaBase)
  private
    OpenCount: Integer;
  public
    Disk: AbsMsgPtr;

    function Opened: Boolean;

    procedure OpenOrCreate;
    procedure Close;
  end;

  TArea = class(TAreaWithDisk)
  protected
    class function Compare(PA, PB: Pointer): Integer; override;
  private
    FCurrent: TLetter;
    SavedCurrentID: Longint;
    FFilteredValid: Boolean;

    procedure SetCurrent(ACurrent: TLetter);

    procedure SaveCurrent;
    procedure RestoreCurrent;

    procedure CollectReplies;
    procedure CMStopUpdatingArea;
    procedure CMAreaCountsChange;
  public
    InfoPtr: Pointer; {  ,   }
    Valid: Boolean;
    Updating: Boolean;
    UnreadCount, JustReadCount: Integer;

    constructor Create(const AID, AComment, AMKID: string);
    destructor Destroy; override;

    procedure Invalidate;
    procedure Update;

    property Current: TLetter read FCurrent write SetCurrent;

    function GetInfo: string;

    procedure MarkAllAsRead;
    procedure CMCurrentLetterChanged;

    function UnreadOrJustReadCount: Integer;
    procedure VisualAdd(Letter: TLetter);
    procedure VisualDelete(Letter, NewCurrent: TLetter);
    procedure VisualChange(Letter: TLetter);
    function LetterByID(ID: Longint): TLetter;
  end;

  TFilteredLetterList = class(TBaseList)
  protected
    class function Compare(PA, PB: Pointer): Integer; override;
  private
  public
  end;

  TConversationIterator = function(Letter: TLetter; InfoPtr: Pointer): Boolean of object;

  TLetter = class
  private
    Area: TArea;
    function GetPhysicalID: Longint;

    procedure InternalMarkAsRead(DoBroadcast: Boolean);
  public
    InfoPtr: Pointer; {  ,   }
    ID: LongInt;

    Number, Reference: Integer;
    Subject, From, WhoTo: string;
    DateTime: TDateTime;
    Replies: TList;
{    FromAddr, ToAddr: AddrType;}
    Unread, JustRead: Boolean;
    MarkedToInclude: Boolean;{,   }

    constructor Create(AArea: TArea;
      ANumber, AReference: Integer;
      const ASubject, AFrom, AWhoTo: string;
      ADateTime: TDateTime;
      AUnread: Boolean);
    destructor Destroy; override;

    function Present: Boolean;

    procedure Seek;
    function IsConversationStart: Boolean;
    function IterateConversation(Func: TConversationIterator; InfoPtr: Pointer): Boolean;

    procedure MarkAsRead;
    function UnreadOrJustRead: Boolean;
    procedure LinkToReference;
    procedure UnlinkFromReference;

    procedure CopyFrom(Source: TLetter);
    procedure Delete;

    procedure ReadText(List: TStrings);
  end;

var
  Areas: TAreas;

implementation

uses
  StrUtils,
  Forms,

  CRC32,
  MKOpen,
  MKString,
  MKMisc,

    mkmsgjam,dialogs,

  Common,
  Commands,

  ComLin,
  Radio,
  Flags;

{ }

function HeaderCRC(const Subject, From, WhoTo: string; DateTime: TDateTime): Longint;
begin
  Result :=
    UpdC32(StrCRC(Subject),
      UpdC32(StrCRC(From),
        UpdC32(StrCRC(WhoTo),
          UpdC32(BufCRC(DateTime, SizeOf(DateTime)),
            $FFFFFFFF))));
end;

{ TBaseList }

procedure TBaseList.FreeAll;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
    TObject(Items[I]).Free;

  Clear;
end;

procedure TBaseList.Sort;
begin
  if Count > 1 then
    QuickSort(0, Count - 1);
end;

procedure TBaseList.QuickSort(L, R: Integer);
var
  I, J: Integer;
  P: Pointer;
begin
  I := L;
  J := R;
  P := Items[(L + R) shr 1];
  repeat
    while Compare(Items[I], P) < 0 do Inc(I);
    while Compare(Items[J], P) > 0 do Dec(J);
    if I <= J then
    begin
      Exchange(I, J);
      Inc(I);
      Dec(J);
    end;
  until I > J;
  if L < J then QuickSort(L, J);
  if I < R then QuickSort(I, R);
end;


{ TAreasBase }

destructor TAreasBase.Destroy;
begin
  FreeAll;

  inherited Destroy;
end;

{ TAreasFromCFG }

constructor TAreasFromCFG.LoadFromCFG;
var
  F: TextFile;
  S: string;
  FEAreaID: string;
  ID, Comment, MKID: string;
begin
  inherited Create;

  AssignFile(F, AreasBBSFileSpec);
  try
    Reset(F);
    while not EOF(F) do
    begin
      ReadLn(F, S);
      if (Length(S) < 2) or
        (S[1] = ';') or
        ((S[1] = 'P') and (S[2]=' ')) then
        Continue;

      FEAreaID := OemToAnsi(ExtractWord(S, 1));
      case FEAreaID[1] of
        '0'..'9':
          MKID := 'H'+PadLeft(FEAreaID, '0', 3)+PathToHudson;
        '$':
          MKID := 'S'+Copy(FEAreaID, 2, $FF);
        '!':
          MKID := 'J'+Copy(FEAreaID, 2, $FF);
        else
          MKID := 'F'+FEareaID;
      end;
      ID := UpperCase(ExtractWord(S, 2));
      Comment := '';

      Add(TAreaBase.Create(ID, Comment, MKID));
    end;
    Close(F);
  except
    on E: EInOutError do
      raise ECantFindAreasBBS.Create(Format(' "%s"?  ( %d)',
        [AreasBBSFileSpec, E.ErrorCode]));
  end;

  Sort;
end;

class function TAreasFromCFG.Compare;
var
  A: TAreaBase absolute PA;
  B: TAreaBase absolute PB;
begin
  if Assigned(A)
  then
    if Assigned(B)
    then Result := AnsiCompareStr(A.ID, B.ID)
    else Result := -1
  else
    Result := +1;
end;


{ TAreas }

constructor TAreas.Create;
begin
  inherited Create;

  FRaw := TList.Create;
end;

destructor TAreas.Destroy;
begin
  FRaw.Free;

  inherited Destroy;
end;

procedure TAreas.CMCurrentAreaChanged;
begin
  Broadcast(CM_CurrentAreaChanged, nil);
end;

procedure TAreas.SetCurrent;
begin
  if FCurrent <> ACurrent then
  begin
    FCurrent := ACurrent;
    CMCurrentAreaChanged;
  end;
end;

procedure TAreas.Invalidate;
begin
  FValid := False;
end;

class function TAreas.Compare;

  function KeyOf(Area: TArea): Longint;
  begin
    with Area do
      if UnreadCount = 0
      then Result := High(Longint)
      else Result := LongInt(UnreadCount);
  end;

var
  A: TArea absolute PA;
  B: TArea absolute PB;
var
  AKey, BKey: Longint;
begin
  if A.Valid < B.Valid
  then
    Result := +1{ Valid  }
  else
    if A.Valid = B.Valid
    then
    begin
      AKey := KeyOf(A);
      BKey := KeyOf(B);
      if AKey < BKey
      then
        Result := -1
      else
        if AKey = BKey
        then Result := AnsiCompareStr(A.ID, B.ID)
        else Result := +1;
    end
    else
      Result := -1;
end;

procedure TAreas.VisibleSort;
begin
  Sort;
  CMCurrentAreaChanged;
  Broadcast(CM_AreasChanged, nil);
end;

procedure TAreas.Update;

  procedure DoUpdate;

    procedure MergeOldAndNewAreas;
    var
      New: TAreasFromCFG;
      Left, Right: Integer;
      LeftArea, RightArea: TArea;
      I: Integer;
    begin
      Clear;

      New := TAreasFromCFG.LoadFromCFG;
      try
        Left := 0;
        Right := 0;

        repeat
          if Left < FRaw.Count
          then LeftArea := FRaw[Left]
          else LeftArea := nil;

          if Right < New.Count
          then RightArea := New[Right]
          else RightArea := nil;

          if not (Assigned(LeftArea) or Assigned(RightArea)) then
            Break;

          case TAreasFromCFG.Compare(LeftArea, RightArea) of
            -1:
              if Assigned(LeftArea) then
              begin
                LeftArea.Free;
                FRaw.Delete(Left);
              end;
            0:
            begin
              LeftArea.CopyFrom(RightArea);
              LeftArea.Invalidate;

              Inc(Left);
              Inc(Right);
            end;
            1:
            begin
              with RightArea do
                FRaw.Insert(Left, TArea.Create(ID, Comment, MKID));
              Inc(Left);
              Inc(Right);
            end;
          end;
        until False;
      finally
        New.Free;
      end;

      for I := 0 to Pred(FRaw.Count) do
        Add(FRaw[I]);

      {Sort;}
      Broadcast(CM_AreasChanged, nil);
    end;

  var
    SaveCurrentID: string;
    I: Integer;
    Area: TArea;
    NewCurrent: TArea;
  begin
    if Assigned(FCurrent)
    then SaveCurrentID := FCurrent.ID
    else SaveCurrentID := '';

    NewCurrent := nil;
    try
      MergeOldAndNewAreas;

      for I := 0 to Pred(Count) do
      begin
        Area := Items[I];
        if Area.ID = SaveCurrentID then
        begin
          NewCurrent := Area;
          Exit;
        end;
      end;

      if Count <> 0 then
        NewCurrent := Items[0];
    finally
      Current := nil;
      Current := NewCurrent;
    end;
  end;

  procedure DoRescan;
  var
    I: Integer;
    Area: TArea;
  begin
    for I := 0 to Pred(Count) do
    begin
      if StopScanning then
        Break;

      Area := Items[I];
      try
        Area.Update;
      except
        {Application.HandleException(nil);}
        (*
        on ECantOpenArea do {Application.HandleException(nil),  !};
        on ECantCloseArea do {Application.HandleException(nil)};*)

        {  !!}
      end;
      Broadcast(CM_NextAreaScanned, Area);
    end;
    VisibleSort;

    if Count = 0
    then Current := nil
    else Current := Items[0];
  end;

begin
  if FValid or Updating then
    Exit;

  Updating := True;
  try
    DoUpdate;

    Broadcast(CM_BeforeScanningAreas, nil);
    try
      DoRescan;
    finally
      Broadcast(CM_AfterScanningAreas, nil);
    end;

    FValid := True;
  finally
    Updating := False;
  end;
end;

function TAreas.GetInfo;
begin
  Result := Format(': %d', [Count]);
end;

function TAreas.GetTotalUnreadCount;
var
  I: Integer;
begin
  Result := 0;
  for I := 0 to Pred(Count) do
    Inc(Result, TArea(Items[I]).UnreadCount);
end;


{ TLetterList }

function TLetterList.LetterByNumber;
var
  I: Integer;
begin
  for I := 0 to Count-1 do
  begin
    Result := TLetter(Items[I]);
    if Result.Number = Number then Exit;
  end;
  Abort{reply linker ,   GoldEd ,   };
end;

{ TAreaBase }

constructor TAreaBase.Create;
begin
  inherited Create;

  ID := AID;
  Comment := AComment;
  MKID := AMKID;
end;

destructor TAreaBase.Destroy;
begin
  inherited Destroy;
end;


procedure TAreaBase.CopyFrom;
begin
  ID := Source.ID;
  Comment := Source.Comment;
  MKID := Source.MKID;
end;


{ TAreaWithDisk }

function TAreaWithDisk.Opened;
begin
  Result := Assigned(Disk);
end;

procedure TAreaWithDisk.OpenOrCreate;
begin
 If not OpenOrCreateMsgArea(Disk, MKID) Then
    raise ECantOpenArea.Create(
      Format('  /  "%s" (%s)', [ID, MKID]));

  Inc(OpenCount);
  Broadcast(CM_AreaOpened, Self);
end;

procedure TAreaWithDisk.Close;
var
  SuccessfullyClosed: Boolean;
begin
  Dec(OpenCount);
  if OpenCount <> 0 then
    Exit;

  SuccessfullyClosed := CloseMsgArea(Disk);
  Disk := nil;
  If not SuccessfullyClosed Then
    raise ECantCloseArea.Create(
      Format('    "%s"', [ID]));

  Broadcast(CM_AreaClosed, Self);
end;


{ TArea }

constructor TArea.Create;
begin
  inherited Create(AID, AComment, AMKID);
end;

destructor TArea.Destroy;
begin
  Broadcast(CM_AreaDestroyed, Self);

  inherited Destroy;
end;


procedure TArea.CMCurrentLetterChanged;
begin
  if not Updating{ ,  Outline  } then
    Broadcast(CM_CurrentLetterChanged, Self);
end;

procedure TArea.SetCurrent;
begin
  if FCurrent <> ACurrent then
  begin
    FCurrent := ACurrent;
    CMCurrentLetterChanged;
  end;
end;

procedure TArea.Invalidate;
begin
  Valid := False;
end;

class function TArea.Compare;
var
  A: TLetter absolute PA;
  B: TLetter absolute PB;

  AD, BD: TDateTime;
begin
  AD := A.DateTime;
  BD := B.DateTime;
  if AD < BD
  then
    Result := -1
  else
    if AD = BD
    then Result := AnsiCompareStr(A.Subject, B.Subject)
    else Result := +1;
end;

procedure TArea.CollectReplies;
var
  I: Integer;
  Letter: TLetter;
begin
  for I := 0 to Pred(Count) do
    {Application.ProcessMessages{!!!}
    TLetter(Items[I]).LinkToReference;
end;

procedure TArea.CMStopUpdatingArea;
begin
  Broadcast(CM_StopUpdatingArea, Self);
end;

procedure TArea.Update;

  procedure DoUpdate;

    function CreateNewList: TLetterList;
    begin
      Result := TLetterList.Create;
      try
        OpenOrCreate;

        try
          with Disk^ do
          begin
            SeekFirst(1);

            while SeekFound Do
            begin
              Application.ProcessMessages;
              if StopScanning then
                Break;

              MsgStartUp;
              Result.Add(TLetter.Create(Self,
                GetMsgNum, GetRefer,
                GetSubj, GetFrom, GetTo,
                GetDateTime,
                not IsRead));

              SeekNext;
            end;
          end;
        finally
          Close;
        end;
      except
        on ECantOpenArea do{};
        on ECantCloseArea do{};
      end;
    end;

    procedure CountTotals;
    var
      I: Integer;
    begin
      UnreadCount := 0;
      JustReadCount := 0;{,    }
      for I := 0 to Pred(Count) do
        with TLetter(Items[I]) do
          if Unread then
            Inc(UnreadCount);
    end;

  var
    New: TList;
    I: Integer;
  begin
    try
      New := CreateNewList;
      try
        FreeAll;
        for I := 0 to Pred(New.Count) do
          Add(New[I]);
        Sort; {   ; }
        CollectReplies;
      finally
        New.Free;
      end;
    finally
      CountTotals;
    end;
  end;

begin
  if Valid or Updating then
    Exit;

  Updating := True;
  SaveCurrent;
  try
    Broadcast(CM_StartUpdatingArea, Self);
    DoUpdate;
    Valid := True;
  finally
    RestoreCurrent;{    ,   Updating=true}
    Updating := False;
    CMStopUpdatingArea;
  end;
end;

procedure TArea.SaveCurrent;
begin
  if Current = nil
  then SavedCurrentID := 0
  else SavedCurrentID := Current.ID;
end;

procedure TArea.RestoreCurrent;
begin
  if UnreadCount = 0
  then Current := nil{          }
  else Current := LetterByID(SavedCurrentID);
end;

function TArea.LetterByID;
var
  I: Integer;
  Letter: TLetter;
begin
  if ID <> 0 then
    for I := 0 to Pred(Count) do
    begin
      Letter := Items[I];
      if Letter.ID = ID then
      begin
        Result := Letter;
        Exit;
      end;
    end;
  Result := nil;
end;

function TArea.GetInfo;
begin
  if Self = nil
  then
    Result := ''
  else
  begin
    Result := ID+': ';
    if Valid
    then
      AppendStr(Result, Format('%d , %d , %d ', [
        UnreadCount, JustReadCount, Count]))
    else
      if Updating
      then AppendStr(Result, '...')
      else AppendStr(Result, ' ');
  end;
end;

procedure TArea.CMAreaCountsChange;
begin
  Broadcast(CM_AreaCountsChange, Self);
end;

procedure TArea.MarkAllAsRead;
var
  I: Integer;
begin
  for I := 0 to Pred(Count) do
    TLetter(Items[I]).InternalMarkAsRead(False);

  CMAreaCountsChange;
end;


function TArea.UnreadOrJustReadCount;
begin
  Result := UnreadCount + JustReadCount;
end;

procedure TArea.VisualAdd;
begin
  Add(Letter);
  Letter.LinkToReference;

  with Letter do
  begin
    if Unread then
      Inc(UnreadCount);
    if JustRead then
      Inc(JustReadCount);
  end;

  VisualChange(Letter);
end;

procedure TArea.VisualDelete;
begin
  Letter.UnlinkFromReference;
  Remove(Letter);

  with Letter do
  begin
    if Unread then
      Dec(UnreadCount);
    if JustRead then
      Dec(JustReadCount);
  end;

  VisualChange(NewCurrent);
end;

procedure TArea.VisualChange;
begin
  Sort;
  FCurrent := Letter;
  CMStopUpdatingArea;
  CMCurrentLetterChanged;

  Broadcast(CM_AreaCountsChange, Self);
end;

{ TFilteredLetterList }

class function TFilteredLetterList.Compare;
var
  A: TLetter absolute PA;
  B: TLetter absolute PB;

  AD, BD: TDateTime;
begin
  Result := AnsiCompareStr(A.Subject, B.Subject);

  if Result = 0 then
  begin
    AD := A.DateTime;
    BD := B.DateTime;
    if AD < BD
    then
      Result := -1
    else
      if AD > BD then
        Result := +1;
  end;
end;


{ TLetter }

constructor TLetter.Create;

  function GetID: Longint;
  begin
    Result := HeaderCRC(Subject, From, Whoto, DateTime);
  end;

begin
  inherited Create;

  Area:= AArea;
  Replies := TList.Create;

  Subject := ASubject;
  From := AFrom;
  WhoTo := AWhoTo;
  DateTime := ADateTime;

  Number := ANumber;
  Reference := AReference;

  Unread := AUnread;

  ID := GetID;
end;

destructor TLetter.Destroy;
begin
  Replies.Free;

  inherited Destroy;
end;

function TLetter.Present;
begin
  Result := GetPhysicalID = ID;
end;

function TLetter.GetPhysicalID;
begin
  {CRC physical header =   , }
  try
    Seek;
  except
    on ELetterNotFound do
    begin
      Result := 0;
      Exit;
    end;
  end;

  with Area.Disk^ do
  begin
    MsgStartUp;
    Result := HeaderCRC(GetSubj, GetFrom, GetTo, GetDateTime);
  end;
end;

procedure TLetter.Seek;
begin
  with Area.Disk^ do
  begin
    SeekFirst(Number);
    if not SeekFound then
      raise ELetterNotFound.Create(Format('   #%d', [Number]));
  end;
end;

function TLetter.IsConversationStart;
begin
  Result := Reference = 0;
end;

function TLetter.IterateConversation;
var
  I: Integer;
begin
  Result := Func(Self, InfoPtr);
  if Result then
    Exit;

  with Replies do
    for I := 0 to Pred(Count) do
      with TLetter(Items[I]) do
      begin
        Result := IterateConversation(Func, InfoPtr);
        if Result then
          Exit;
      end;
end;

procedure TLetter.InternalMarkAsRead;
begin
  if not Unread then
    Exit;

  Seek;
  with Area, Disk^ do
  begin
    MsgStartUp;
    SetRead(True);
    ReWriteHdr;
{   lock'}

    Unread := False;  Dec(UnreadCount);
    JustRead := True;  Inc(JustReadCount);
  end;

  if DoBroadcast then
    Area.CMAreaCountsChange;
end;

procedure TLetter.MarkAsRead;
begin
  InternalMarkAsRead(True);
end;

function TLetter.UnreadOrJustRead;
begin
  Result := Unread or JustRead;
end;

procedure TLetter.LinkToReference;
begin
  if Reference <> 0 then
    try
      Area.LetterByNumber(Reference).Replies.Add(Self);
    except
      on EAbort{  } do
        Reference := 0{};
    end;
end;

procedure TLetter.UnlinkFromReference;
begin
  if Reference <> 0 then
    Area.LetterByNumber(Reference).Replies.Remove(Self);
end;

procedure TLetter.CopyFrom;
begin
  Number := Source.Number;
  ID := Source.ID;
  From := Source.From;
  WhoTo := Source.WhoTo;
  Subject := Source.Subject;
  DateTime := Source.DateTime;
end;

procedure TLetter.Delete;
begin
  Seek;

  Area.Disk^.DeleteMsg;
end;

procedure TLetter.ReadText;
begin
  with Area.Disk^, List do
  begin
    MsgTxtStartUp;

    While Not EOM Do
      Add(Tab2Space(GetString({80}$FFFF){ ,   LastSoft,
              False, ..   ,
                ,
            . ReFormat.
          , ..  ,    .}, 8));

    while Count <> 0 do
      if Strings[Pred(Count)] = ''
      then Delete(Pred(Count))
      else Break;
  end;
end;


{  }

procedure MailUnitDone;
begin
  Areas.Free;
end;

begin
  Areas := TAreas.Create;

  AddExitProc(MailUnitDone);
end.
