unit ReFormat;

interface

uses
  Classes,
  Graphics,

  MKMsgAbs,
  MKGlobT;

type
  TFormattedStrings = class(TStringList)
  private
    Origin: string;
  public
    Kludges: TStrings;
    MsgID, Reply: string;
    FromAddr, ToAddr: AddrType;
    procedure AnalizeAndReFormat(
      IncludeKludges: Boolean;
      AWrapCharWidth: Integer;
      AWrapDotWidth: Integer; Canvas: TCanvas;
      Disk: AbsMsgPtr);
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils,

  StrUtils,
  MKString,

  Common,

  TextKind;

procedure TFormattedStrings.AnalizeAndReFormat;
const
  KludgeChar = ^A;

  function FindCharWidth(const S: string; WrapDotWidth: Integer; Canvas: TCanvas): Integer;
  var
    Low, Middle, High: Integer;
  begin
    Result := $FF;
    if Canvas.TextWidth(S) < WrapDotWidth then
      Exit;

    Low := 1;
    High := Length(S);
    while Low < High do
    begin
      Middle := (Low+High) div 2;
      if Canvas.TextWidth(Copy(S, 1, Middle)) < WrapDotWidth
      then Low := Middle+1
      else High := Middle-1;
    end;
    Result := Low-1;
  end;

  function QuoteChars(const S: string): string;
  var
    P: Integer;
  begin
    P := QuotePos(S);
    Result := Copy(S, 1, P);
  end;

  function InsertPos(const S: string): Integer;
  begin
    Result := QuotePos(S);
    if Result < 0 then
      Exit;

    Inc(Result);
    if Copy(S, Result, 1) = ' '{     } then
      Inc(Result);
  end;

  procedure Parse;
  begin
    if (MsgID = '') or not ExtractAddress(ExtractWord(MsgID, 1), FromAddr) then
      if (Origin = '') or not ExtractAddress(Origin, FromAddr) then
        Disk^.GetOrig(FromAddr);

    if (Reply = '') or not ExtractAddress(ExtractWord(Reply, 1), ToAddr) then
      Disk^.GetDest(ToAddr);
  end;

var
  WrapCharWidth, WrapDotWidth: Integer;
  I: Integer;
  S, Wrap, NextLine: string;
  MyQuoteChars: string;
  Kludge: Boolean;
  P: Integer;
const
  MinWrapCharWidth = Succ(LastQuotePos)*2;
  MinWrapDotWidth = Succ(LastQuotePos)*2* 10;
begin
  if AWrapCharWidth <> 0 then
    if AWrapCharWidth < MinWrapCharWidth then
      AWrapCharWidth := MinWrapCharWidth;

  if AWrapDotWidth <> 0 then{    <> 0 ,   }
    if AWrapDotWidth < MinWrapDotWidth then
      AWrapDotWidth := MinWrapDotWidth;

  I := 0;
  while I < Count do
  begin
    S := Strings[I];

    if IsOrigin(S) then
      Origin := S;

    if IsOrigin(S) or IsTearLine(S){  wrap'} then
    begin
      Inc(I);
      Continue;
    end;

    Kludge := IsKludge(S);
    if Kludge then
    begin
      if HasStart(S, ^A'MSGID:')
      then
        MsgID := Copy(S, 8, $FF)
      else
        if HasStart(S, ^A'REPLY:')
        then
          Reply := Copy(S, 8, $FF)
        else
          Kludges.Add(S);
    end;

    if Kludge and not IncludeKludges then
    begin
      Delete(I);
      Continue;
    end;

    WrapCharWidth := AWrapCharWidth;
    WrapDotWidth := AWrapDotWidth;

    P := QuotePos(S);
    if P > 0 then
    begin
      if Copy(S, Succ(P), 1) = ' ' then
        Inc(P);
      MyQuoteChars := Copy(S, 1, P);
      System.Delete(S, 1, P);
      if Assigned(Canvas)
      then Dec(WrapDotWidth, Canvas.TextWidth(MyQuoteChars))
      else Dec(WrapCharWidth, P);
    end;

    if Assigned(Canvas) then
      WrapCharWidth := FindCharWidth(S, WrapDotWidth, Canvas);

    WordWrap(S, S, Wrap, WrapCharWidth); {? !}
    if P > 0 then
    begin
      S := MyQuoteChars + S;
      if Assigned(Canvas)
      then Inc(WrapDotWidth, Canvas.TextWidth(MyQuoteChars))
      else Inc(WrapCharWidth, Pred(P));
    end;

    if Wrap <> '' then
    begin
      Strings[I] := S;

      if IsKludge(S) then
      begin
        Insert(I+1, ^A' '+Wrap);
      end
      else
      begin
        if I+1 < Count
        then NextLine := Strings[I+1]
        else NextLine := '';
        MyQuoteChars := QuoteChars(S);
        if MyQuoteChars = QuoteChars(NextLine)
        then
          if (MyQuoteChars = '') and (
            (NextLine = '') or (IsTearline(NextLine) or IsOrigin(NextLine)))
          then
            Insert(I+1, Wrap)
          else
          begin
            System.Insert(Wrap+' ', NextLine, InsertPos(NextLine)) {  quote};
            if I+1 < Count
            then Strings[I+1] := NextLine
            else Insert(I+1, NextLine);
          end
        else
        begin
          if MyQuoteChars <> '' then
            AppendStr(MyQuoteChars, ' ');

          Insert(I+1, MyQuoteChars+Wrap);
        end;
      end;
    end;
    Inc(I);
  end;

  if Assigned(Disk) then
    Parse;
end;

constructor TFormattedStrings.Create;
begin
  inherited Create;
  Kludges := TStringList.Create;
end;

destructor TFormattedStrings.Destroy;
begin
  Kludges.Free;

  inherited Destroy;
end;

end.

