unit TextKind;

interface

uses
  Graphics;

const
  LastQuotePos = 7;
  QuoteChars = ['>'{, ':'   }];

type
  TTextKind = (tkNormal, tkQuote, tkTearline, tkOrigin, tkKludge, tkWhitespace);

const
  tkColors: array[TTextKind] of TColor = (
//  (tkNormal, tkQuote, tkTearline, tkOrigin, tkKludge, tkWhitespace);
    clBlack,  clNavy, clSilver, clSilver, clRed, clBlack);

function HasStart(const S, Start: string): Boolean;

function QuotePos(const S: string): Integer;
function IsTearline(const S: string): Boolean;
function IsOrigin(const S: string): Boolean;
function IsKludge(const S: string): Boolean;
function IsSeenBy(const S: string): Boolean;

function GetTextKind(const S: string): TTextKind;

implementation

function QuotePos;
var
  TempS: string;
  I: Integer;
begin
  Result := -1;
  TempS := Copy(S, 1, LastQuotePos);
  for I := Length(TempS) downto 1 do
    if TempS[I] in QuoteChars then
    begin
      Result := I;
      break;
    end;
end;

function HasStart;
begin
  Result := Copy(S, 1, Length(Start)) = Start;
end;

function IsKludge;
begin
  Result := HasStart(S, ^A);
end;

function IsSeenBy;
begin
  Result := HasStart(S, 'SEEN-BY:');
end;

function IsTearline;
const
  TearlineStart = '---';
begin
  Result := (S = TearlineStart) or HasStart(S, TearlineStart+' ');
end;

function IsOrigin;
begin
  Result := HasStart(S, ' * Origin:');
end;

function GetTextKind;
begin
  if QuotePos(S) >= 1
  then
    Result := tkQuote
  else
    if IsTearline(S)
    then
      Result := tkTearline
    else
      if IsOrigin(S)
      then
        Result := tkOrigin
      else
        if IsKludge(S) or IsSeenBy(S)
        then Result := tkKludge
        else Result := tkNormal;
end;

end.
